class MyGame extends Game {
  public static final int SCENE_TITLE = 0, SCENE_GAME = 1, 
  SCENE_GAMEOVER = 2, SCENE_GAMECLEAR = 3;
  private int scene;
  private PImage titleImage, subTitleImage, pressKeyImage, gameOverImage, gameClearImage, oniImage;
  private boolean fade = false;
  private float pressedAt;
  private float frequency = 5000;
  private PImage light = loadImage("light.png");
  public static final int MAX_BEANS = 30;
  public boolean beansFlag;
  
  CreateMap _createMap;
  Player _player;
  Oni _oni;
  Beans[] beans;
  
  public MyGame( PApplet applet ) {
    super( applet );
  }
 
  protected void initialize() {
    scene = SCENE_TITLE;
    titleImage = loadImage( "title.png" );
    subTitleImage = loadImage( "subTitle.png" );
    pressKeyImage = loadImage( "pressSpaceKey.png" );
    gameOverImage = loadImage( "gameOver.png" );
    gameClearImage = loadImage( "gameClear.png" );
    oniImage = loadImage( "oni.png" );
    imageMode( CENTER );
    _createMap = new CreateMap();
    _player = new Player(this);
    _oni = new Oni(this);
    beans = new Beans[MAX_BEANS];
    for(int i = 0; i < MAX_BEANS; i++){
      beans[i] = new Beans( this );
    }
    _createMap.initialize();
    _player.initialize(1,1);
    _oni.initialize(10,10);
    titleSound();
    beansFlag = true;
    super.initialize();
  }
  
  protected void update() {
    background(0);
    switch( scene )
    {
    case SCENE_TITLE:
      title();
      break;

    case SCENE_GAME:
      game();
      break;

    case SCENE_GAMEOVER:
      gameOver();
      break;

    case SCENE_GAMECLEAR:
      gameClear();
      break;
    }
    if ( fade ) {
      fill( 0, - 128*cos( TWO_PI/frequency*(millis()-pressedAt)) + 128);
      rect( 0, 0, width, height );
      if (millis()-pressedAt >= frequency) {
        fade = false;
      }
    }
  }
  
  private void title()
  {
    tint(255);
    image( oniImage, width, height / 2 );
    image( titleImage, width / 3, height / 3.1 );
    image( subTitleImage, width / 3, height / 2.1 );
    if ( KEY_SPACE && fade == false)
    {
      setMsec();
      selectSound();
    } else {
      float blinkTimer = millis();
      tint( 255, 128*sin( TWO_PI/frequency*blinkTimer) + 128 );
      image( pressKeyImage, width / 3, height / 1.3 );
    }

    if ((millis() - pressedAt >= frequency / 2) && fade ) {
      scene = SCENE_GAME;
    }
  }

  private void setMsec() {
    fade = true;
    pressedAt = millis();
  }

  private void game() 
  {
    pushMatrix();
    translate((float)(-_player.position.x + width / 2),(float)(-_player.position.y+ height / 2));
    _createMap.drawMapChip();
    _player.update(delta);
    _oni.update(delta);
    for(int i = 0; i < MAX_BEANS; i++){
      beans[i].update(delta);
    }
    tint(255);
    image(light,(float)(_player.position.x - WIDTH/2),(float)(_player.position.y - HEIGHT/2));
    popMatrix();
    if(_player.beansCount == MAX_BEANS){
      scene = SCENE_GAMEOVER;
    }
    if(_createMap.map[_oni._dy][_oni._dx] == 16){
      scene = SCENE_GAMECLEAR;
    }
  }

  private void gameOver()
  {
    image( oniImage, width / 2, height / 2 );
    image( gameOverImage, width / 2, height / 2.5 );
    if ( KEY_SPACE ) {
      setMsec();
    } else {
      if ((millis()-pressedAt >= frequency /2) && fade) {
        scene = SCENE_TITLE;
      }
    }
  }


  private void gameClear()
  {
    image( gameClearImage, width / 2, height / 2.5 );
    image( pressKeyImage, width / 2, height / 1.3 );
    if ( KEY_SPACE ) {
      setMsec();
    } else {
      if ((millis()-pressedAt >= frequency/2) && fade) {
        scene = SCENE_TITLE;
      }
    }
  }
  
}
