class Oni extends BaseCharacter {
  public static final int STATE_WAIT = 0;
  public static final int STATE_MOVE_TO_RIGHT = 1;
  public static final int STATE_MOVE_TO_LEFT = 2;
  public static final int STATE_MOVE_TO_UP = 3;
  public static final int STATE_MOVE_TO_DOWN = 4;
  public static final int STATE_MOVE_TO_DASH_RIGHT = 5;
  public static final int STATE_MOVE_TO_DASH_LEFT = 6;
  public static final int STATE_MOVE_TO_DASH_UP = 7;
  public static final int STATE_MOVE_TO_DASH_DOWN = 8;
  public static final int STATE_HIT = 9;

  public static final int ANIM_MOVE_TO_LEFT = 1;
  public static final int ANIM_MOVE_TO_RIGHT = 0;
  public static final int ANIM_MOVE_TO_UP = 2;
  public static final int ANIM_MOVE_TO_DOWN = 3;
  public static final int ANIM_MOVE_TO_DASH_LEFT = 6;
  public static final int ANIM_MOVE_TO_DASH_RIGHT = 4;
  public static final int ANIM_MOVE_TO_DASH_UP = 7;
  public static final int ANIM_MOVE_TO_DASH_DOWN = 5;

  public int _state;
  private int speed = 40;
  private int waitCount;
  private int fx = 0;
  private int fy = 0;
  public boolean hitFlag;

  private int moveCount;

  private int moveWay;
  private float _x, _y;
  private int _dx, _dy;
  public static final int WIDTH = 128;
  public static final int HEIGHT = 128;
  private final int IMAGE_WIDTH = 4;
  private final int IMAGE_HEIGHT = 6;

  public Oni( Game parent ) {
    super( parent, "oniAnim.png", 8, 8 );
    _state = STATE_WAIT;
    waitCount = 0;
    moveCount = 0;
    hitFlag = false;
  }

  public void initialize( int dx, int dy ) {
    MyGame theParent = (MyGame)_parent;
    _dx = dx;
    _dy = dy;
    _x = ( _dx * WIDTH ) + WIDTH /2  - WIDTH * (theParent._createMap.map[0].length / 2 )+ IMAGE_WIDTH * WIDTH;
    _y = ( _dy * HEIGHT ) + HEIGHT /2 - HEIGHT * (theParent._createMap.map.length / 2) + IMAGE_HEIGHT * HEIGHT;
    position.set( _x, _y );
  }

  public void update( double deltaTime ) {
    switch( _state ) {
    case STATE_WAIT:
      _state = waitInput();
      break;
    case STATE_MOVE_TO_RIGHT:
      _state = moveToRight();
      break;
    case STATE_MOVE_TO_LEFT:
      _state = moveToLeft();
      break;
    case STATE_MOVE_TO_UP:
      _state = moveToUp();
      break;
    case STATE_MOVE_TO_DOWN:
      _state = moveToDown();
      break;
    case STATE_MOVE_TO_DASH_RIGHT:
      _state = moveToDashRight();
      break;
    case STATE_MOVE_TO_DASH_LEFT:
      _state = moveToDashLeft();
      break;
    case STATE_MOVE_TO_DASH_UP:
      _state = moveToDashUp();
      break;
    case STATE_MOVE_TO_DASH_DOWN:
      _state = moveToDashDown();
      break;
    case STATE_HIT:
      break;
    }


    //------
    super.update( deltaTime );
  }

  private int waitInput() {
    int result = _state;
    MyGame theParent = (MyGame)_parent;

    if (moveCount == 0) {
      if (this._dx + 2 == theParent._player._dx && this._dy == theParent._player._dy) {
        result = STATE_MOVE_TO_DASH_RIGHT;
        return result;
      }
      if (this._dx - 2 == theParent._player._dx && this._dy == theParent._player._dy) {
        result = STATE_MOVE_TO_DASH_LEFT;
        return result;
      }
      if (this._dy - 2 == theParent._player._dy && this._dx == theParent._player._dx) {
        result = STATE_MOVE_TO_DASH_UP;
        return result;
      }
      if (this._dy + 2 == theParent._player._dy && this._dx == theParent._player._dx) {
        result = STATE_MOVE_TO_DASH_DOWN;
        return result;
      }
      moveWay = int(random(1, 5));

      if (moveWay == STATE_MOVE_TO_RIGHT) {
        if ( theParent._createMap.map[_dy][_dx + 1] <= 1 || theParent._createMap.map[_dy][_dx + 1] >= 17) {
          result = STATE_WAIT;
        } else {
          result = STATE_MOVE_TO_RIGHT;
        }
      } else if (moveWay == STATE_MOVE_TO_LEFT) {
        if ( theParent._createMap.map[_dy][_dx - 1] <= 1 || theParent._createMap.map[_dy][_dx - 1] >= 17) {
          result = STATE_WAIT;
        } else {
          result = STATE_MOVE_TO_LEFT;
        }
      } else if (moveWay == STATE_MOVE_TO_UP) {
        if ( theParent._createMap.map[_dy - 1][_dx] <= 1 || theParent._createMap.map[_dy - 1][_dx] >= 17) {
          result = STATE_WAIT;
        } else {
          result = STATE_MOVE_TO_UP;
        }
      } else if (moveWay == STATE_MOVE_TO_DOWN) {
        if ( theParent._createMap.map[_dy + 1][_dx] <= 1 || theParent._createMap.map[_dy + 1][_dx] >= 17) {
          result = STATE_WAIT;
        } else {
          result = STATE_MOVE_TO_DOWN;
        }
      }
    }

    return result;
  } 


  private int moveToRight() {
    int result = _state;
    MyGame theParent = (MyGame)_parent;
    changeAnimation( ANIM_MOVE_TO_RIGHT );
    if (moveCount <= 128) {
      _x = (_dx * WIDTH)  - WIDTH * (theParent._createMap.map[0].length / 2 )+ IMAGE_WIDTH * WIDTH + moveCount + WIDTH/2;
      moveCount++;
    } else {
      _dx++;
      result = STATE_WAIT;
      moveCount = 0;
    }
    position.x = _x;
    position.y = _y;
    return result;
  }

  private int moveToLeft() {
    int result = _state;
    MyGame theParent = (MyGame)_parent;
    changeAnimation( ANIM_MOVE_TO_LEFT );
    if (moveCount <= 128) {
      _x = (_dx * WIDTH)  - WIDTH * (theParent._createMap.map[0].length / 2 )+ IMAGE_WIDTH * WIDTH - moveCount + WIDTH/2;
      moveCount++;
    } else {
      _dx--;
      result = STATE_WAIT;
      moveCount = 0;
    }
    position.x = _x;
    position.y = _y;
    return result;
  }

  private int moveToUp() {
    int result = _state;
    MyGame theParent = (MyGame)_parent;
    changeAnimation( ANIM_MOVE_TO_UP );
    if (moveCount <= 128) {
      _y = (_dy * HEIGHT) - HEIGHT * (theParent._createMap.map.length / 2) + IMAGE_HEIGHT * HEIGHT - moveCount  + WIDTH/2;
      moveCount++;
    } else {
      _dy--;
      result = STATE_WAIT;
      moveCount = 0;
    }
    position.x = _x;
    position.y = _y;
    return result;
  }

  private int moveToDown() {
    int result = _state;
    MyGame theParent = (MyGame)_parent;
    changeAnimation( ANIM_MOVE_TO_DOWN );
    if (moveCount <= 128) {
      _y = (_dy * HEIGHT) - HEIGHT * (theParent._createMap.map.length / 2) + IMAGE_HEIGHT * HEIGHT + moveCount  + WIDTH/2;
      moveCount++;
    } else {
      _dy++;    
      result = STATE_WAIT;
      moveCount = 0;
    }
    position.x = _x;
    position.y = _y;
    return result;
  }

  private int moveToDashRight() {
    int result = _state;
    MyGame theParent = (MyGame)_parent;
    if ( theParent._createMap.map[_dy][_dx + 1] >= 2 || theParent._createMap.map[_dy][_dx + 1] <= 15) {
      if ( theParent._createMap.map[_dy][_dx + 2] >= 2 || theParent._createMap.map[_dy][_dx + 2] <= 15) {
        changeAnimation( ANIM_MOVE_TO_DASH_RIGHT );
        if (moveCount <= 128) {
          _x = (_dx * WIDTH)  - WIDTH * (theParent._createMap.map[0].length / 2 )+ IMAGE_WIDTH * WIDTH + moveCount*2 + WIDTH/2;
          moveCount++;
        } else {
          _dx += 2;
          result = STATE_WAIT;
          moveCount = 0;
        }
        position.x = _x;
        position.y = _y;
      }
    } else {
      result = STATE_WAIT;
    }
    return result;
  }

  private int moveToDashLeft() {
    int result = _state;
    MyGame theParent = (MyGame)_parent;
    if ( theParent._createMap.map[_dy][_dx - 1] >= 2 || theParent._createMap.map[_dy][_dx - 1] <= 15) {
      if ( theParent._createMap.map[_dy][_dx - 2] >= 2 || theParent._createMap.map[_dy][_dx - 2] <= 15) {
        changeAnimation( ANIM_MOVE_TO_DASH_LEFT );
        if (moveCount <= 128) {
          _x = (_dx * WIDTH)  - WIDTH * (theParent._createMap.map[0].length / 2 )+ IMAGE_WIDTH * WIDTH - moveCount*2 + WIDTH/2;
          moveCount++;
        } else {
          _dx-=2;
          result = STATE_WAIT;
          moveCount = 0;
        }
        position.x = _x;
        position.y = _y;
      }
    } else {
      result = STATE_WAIT;
    }
    return result;
  }


  private int moveToDashUp() {
    int result = _state;
    MyGame theParent = (MyGame)_parent;
    if ( theParent._createMap.map[_dy - 1][_dx] >= 2 || theParent._createMap.map[_dy - 1][_dx] <= 15) {
      if ( theParent._createMap.map[_dy - 2][_dx] >= 2 || theParent._createMap.map[_dy - 2][_dx] <= 15) {
        changeAnimation( ANIM_MOVE_TO_DASH_UP );
        if (moveCount <= 128) {
          _y = (_dy * HEIGHT) - HEIGHT * (theParent._createMap.map.length / 2) + IMAGE_HEIGHT * HEIGHT - moveCount*2  + WIDTH/2;
          moveCount++;
        } else {
          _dy-=2;
          result = STATE_WAIT;
          moveCount = 0;
        }
        position.x = _x;
        position.y = _y;
      }
    } else {
      result = STATE_WAIT;
    }
    return result;
  }

  private int moveToDashDown() {
    int result = _state;
    MyGame theParent = (MyGame)_parent;
    if ( theParent._createMap.map[_dy + 1][_dx] >= 2 || theParent._createMap.map[_dy + 1][_dx] <= 15) {
      if ( theParent._createMap.map[_dy + 2][_dx] >= 2 || theParent._createMap.map[_dy + 2][_dx] <= 15) {
        changeAnimation( ANIM_MOVE_TO_DASH_DOWN );
        if (moveCount <= 128) {
          _y = (_dy * HEIGHT) - HEIGHT * (theParent._createMap.map.length / 2) + IMAGE_HEIGHT * HEIGHT + moveCount*2  + WIDTH/2;
          moveCount++;
        } else {
          _dy+=2;    
          result = STATE_WAIT;
          moveCount = 0;
        }
        position.x = _x;
        position.y = _y;
      }
    } else {
      result = STATE_WAIT;
    }
    return result;
  }
  public void hit(int moveWay) {
    _state = STATE_HIT;
    MyGame theParent = (MyGame)_parent;

    if (moveWay == 1) {
      changeAnimation( ANIM_MOVE_TO_DASH_DOWN );
      if (moveCount <= 128) {
        _y = (_dy * HEIGHT) - HEIGHT * (theParent._createMap.map.length / 2) + IMAGE_HEIGHT * HEIGHT + moveCount*2  + WIDTH/2;
        moveCount++;
      } else {
        _dy+=2;    
        _state = STATE_WAIT;
        moveCount = 0;
      }
      position.x = _x;
      position.y = _y;
    } else if (moveWay == 0) {
      changeAnimation( ANIM_MOVE_TO_DASH_UP );
      if (moveCount <= 128) {
        _y = (_dy * HEIGHT) - HEIGHT * (theParent._createMap.map.length / 2) + IMAGE_HEIGHT * HEIGHT - moveCount*2  + WIDTH/2;
        moveCount++;
      } else {
        _dy-=2;
        _state = STATE_WAIT;
        moveCount = 0;
      }
      position.x = _x;
      position.y = _y;
    } else if (moveWay == 2) {
      changeAnimation( ANIM_MOVE_TO_DASH_RIGHT );
      if (moveCount <= 128) {
        _x = (_dx * WIDTH)  - WIDTH * (theParent._createMap.map[0].length / 2 )+ IMAGE_WIDTH * WIDTH + moveCount*2 + WIDTH/2;
        moveCount++;
      } else {
        _dx += 2;
        _state = STATE_WAIT;
        moveCount = 0;
      }
      position.x = _x;
      position.y = _y;
    } else if (moveWay == 3) {
      changeAnimation( ANIM_MOVE_TO_DASH_LEFT );
      if (moveCount <= 128) {
        _x = (_dx * WIDTH)  - WIDTH * (theParent._createMap.map[0].length / 2 )+ IMAGE_WIDTH * WIDTH - moveCount*2 + WIDTH/2;
        moveCount++;
      } else {
        _dx-=2;
        _state = STATE_WAIT;
        moveCount = 0;
      }
      position.x = _x;
      position.y = _y;
    } else if (moveWay == 4) {
      changeAnimation( ANIM_MOVE_TO_DASH_UP );
      if (moveCount <= 128) {
        _y = (_dy * HEIGHT) - HEIGHT * (theParent._createMap.map.length / 2) + IMAGE_HEIGHT * HEIGHT + moveCount*2  + WIDTH/2;
        moveCount++;
      } else {
        _dy-=2;
        _state = STATE_WAIT;
        moveCount = 0;
      }
      position.x = _x;
      position.y = _y;
    }
  }
}

