class Player extends BaseCharacter {
  public static final int STATE_WAIT = 0;
  public static final int STATE_MOVE_TO_RIGHT = 1;
  public static final int STATE_MOVE_TO_LEFT = 2;
  public static final int STATE_MOVE_TO_UP = 3;
  public static final int STATE_MOVE_TO_DOWN = 4;
  public static final int STATE_MOVE_TO_URIGHT = 5;
  public static final int STATE_MOVE_TO_ULEFT = 6;
  public static final int STATE_MOVE_TO_DRIGHT = 7;
  public static final int STATE_MOVE_TO_DLEFT = 8;
  public static final int STATE_THROW = 9;

  public static final int ANIM_MOVE_TO_LEFT = 2;
  public static final int ANIM_MOVE_TO_RIGHT = 3;
  public static final int ANIM_MOVE_TO_UP = 0;
  public static final int ANIM_MOVE_TO_DOWN = 1;  

  public int beansCount = 0;
  private int moveCount;  
  public int direction = 0;
  private int speed = 3;
  private int _state;
  private int _dx, _dy;
  private float _x, _y;
  public static final int WIDTH = 128;
  public static final int HEIGHT = 128;
  private final int IMAGE_WIDTH = 4;
  private final int IMAGE_HEIGHT = 6;


  public Player( Game parent ) {
    super( parent, "player.png", 8, 4 );
    _state = STATE_WAIT;
    moveCount = 0;
  }

  public void initialize(int dx, int dy) {
    MyGame theParent = (MyGame)_parent;
    _dx = dx;
    _dy = dy;
    _x = ( _dx * WIDTH ) + WIDTH / 2  - WIDTH * (theParent._createMap.map[0].length / 2 )+ IMAGE_WIDTH * WIDTH;
    _y = ( _dy * HEIGHT ) + HEIGHT / 2 - HEIGHT * (theParent._createMap.map.length / 2) + IMAGE_HEIGHT * HEIGHT;
    position.set(_x, _y);
  }

  public void update( double deltaTime ) {
    switch( _state ) {
    case STATE_WAIT:
      _state = waitInput();
      break;
    case STATE_MOVE_TO_RIGHT:
      _state = moveToRight();
      break;
    case STATE_MOVE_TO_LEFT:
      _state = moveToLeft();
      break;
    case STATE_MOVE_TO_UP:
      _state = moveToUp();
      break;
    case STATE_MOVE_TO_DOWN:
      _state = moveToDown();
      break;
    case STATE_THROW:
      _state = throwBeans();
      break;
    }

    //------
    super.update( deltaTime );
  }




  private int waitInput() {
    int result = _state;
    MyGame theParent = (MyGame)_parent;
    changeAnimation( 1 );
    if (KEY_RIGHT) {
      if ( theParent._createMap.map[_dy][_dx + 1] <= 1 || theParent._createMap.map[_dy][_dx + 1] >= 16) {
        result = STATE_WAIT;
      } else {
        result = STATE_MOVE_TO_RIGHT;
      }
    } else if (KEY_LEFT) {
      if ( theParent._createMap.map[_dy][_dx - 1] <= 1 || theParent._createMap.map[_dy][_dx - 1] >= 16) {
        result = STATE_WAIT;
      } else {
        result = STATE_MOVE_TO_LEFT;
      }
    } else if (KEY_UP) {
      if ( theParent._createMap.map[_dy - 1][_dx] <= 1 || theParent._createMap.map[_dy - 1][_dx] >= 16) {
        result = STATE_WAIT;
      } else {
        result = STATE_MOVE_TO_UP;
      }
    } else if (KEY_DOWN) {
      if ( theParent._createMap.map[_dy + 1][_dx] <= 1 || theParent._createMap.map[_dy + 1][_dx] >= 16) {
        result = STATE_WAIT;
      } else {
        result = STATE_MOVE_TO_DOWN;
      }
    } else if (KEY_SPACE) {
      result = STATE_THROW;
    }
    return result;
  } 

  private int moveToRight() {
    int result = _state;
    MyGame theParent = (MyGame)_parent;
    changeAnimation( ANIM_MOVE_TO_RIGHT );
    if (moveCount <= 128) {
      _x = (_dx * WIDTH)  - WIDTH * (theParent._createMap.map[0].length / 2 )+ IMAGE_WIDTH * WIDTH + moveCount + WIDTH/2;
      moveCount++;
    } else {
      _dx++;
      result = STATE_WAIT;
      moveCount = 0;
    }
    direction = STATE_MOVE_TO_RIGHT;
    position.x = _x;
    position.y = _y;
    return result;
  }

  private int moveToLeft() {
    int result = _state;
    MyGame theParent = (MyGame)_parent;
    changeAnimation( ANIM_MOVE_TO_LEFT );
    if (moveCount <= 128) {
      _x = (_dx * WIDTH)  - WIDTH * (theParent._createMap.map[0].length / 2 )+ IMAGE_WIDTH * WIDTH - moveCount + WIDTH/2;
      moveCount++;
    } else {
      _dx--;
      result = STATE_WAIT;
      moveCount = 0;
    }
    direction = STATE_MOVE_TO_LEFT;
    position.x = _x;
    position.y = _y;
    return result;
  }

  private int moveToUp() {
    int result = _state;
    MyGame theParent = (MyGame)_parent;
    changeAnimation( ANIM_MOVE_TO_UP );
    if (moveCount <= 128) {
      _y = (_dy * HEIGHT) - HEIGHT * (theParent._createMap.map.length / 2) + IMAGE_HEIGHT * HEIGHT - moveCount  + WIDTH/2;
      moveCount++;
    } else {
      _dy--;
      result = STATE_WAIT;
      moveCount = 0;
    }
    direction = STATE_MOVE_TO_UP;
    position.x = _x;
    position.y = _y;
    return result;
  }

  private int moveToDown() {
    int result = _state;
    MyGame theParent = (MyGame)_parent;
    changeAnimation( ANIM_MOVE_TO_DOWN );
    if (moveCount <= 128) {
      _y = (_dy * HEIGHT) - HEIGHT * (theParent._createMap.map.length / 2) + IMAGE_HEIGHT * HEIGHT + moveCount  + WIDTH/2;
      moveCount++;
    } else {
      _dy++;    
      result = STATE_WAIT;
      moveCount = 0;
    }
    direction = STATE_WAIT;
    position.x = _x;
    position.y = _y;
    return result;
  }

  private int throwBeans() {
    MyGame theParent = (MyGame)_parent;
    int result = _state;
    if (beansCount <= 30 && theParent.beansFlag) {
      switch(direction) {
      case STATE_MOVE_TO_UP:
        theParent.beans[beansCount].initialize( _dy , _dx ,0);
        break;
      case STATE_MOVE_TO_DOWN:
        theParent.beans[beansCount].initialize( _dy , _dx ,1);
        break;
      case STATE_MOVE_TO_RIGHT:
        theParent.beans[beansCount].initialize( _dy, _dx ,2);
        break;
      case STATE_MOVE_TO_LEFT:
        theParent.beans[beansCount].initialize( _dy, _dx ,3);
        break;
      case STATE_WAIT:
        theParent.beans[beansCount].initialize( _dy, _dx ,4);
        break;
      default:
        println("default");
      }

      beansCount++;
      theParent.beansFlag = false;
    }
    result = STATE_WAIT;
    return result;
  }
}

